""" Main api routes for inSPIRE-interactive
"""
from argparse import ArgumentParser
import os
import shutil
import socket
import time

import flask
from flask import request, Response, render_template, send_file, send_from_directory
from flask.json import jsonify
from flask_cors import cross_origin
from jinja2.exceptions import TemplateNotFound
import pandas as pd
import yaml

from inspire_interact.clean_up import (
    clear_queue,
    cancel_job_helper,
)
from inspire_interact.constants import (
    ALL_CONFIG_KEYS,
    CPUS_KEY,
    FRAGGER_MEMORY_KEY,
    FRAGGER_PATH_KEY,
    KEY_FILES,
    MHCPAN_KEY,
    INTERACT_HOME_KEY,
    MODE_KEY,
    QUEUE_PATH,
    RESCORE_COMMAND_KEY,
    SERVER_ADDRESS_KEY,
    SKYLINE_RUNNER_KEY,
    ZIP_PATHS,
)
from inspire_interact.handle_results import (
    create_queue_fig,
    deal_with_failure,
    deal_with_queue,
    deal_with_success,
    deal_with_waiting,
    fetch_queue_and_task,
    safe_fetch,
)
from inspire_interact.inspire_execute import execute_inspire
from inspire_interact.utils import (
    check_pids,
    generate_raw_file_table,
    format_header_and_footer,
    read_meta,
)

app = flask.Flask(__name__, template_folder='templates')


@app.route('/favicon.ico')
@cross_origin()
def favicon():
    """ Endpoint sending the inSPIRE favicon.
    """
    return send_from_directory(
        os.path.join(app.root_path, 'static'),
        'favicon.ico',
        mimetype='image/vnd.microsoft.icon',
    )


@app.route('/static/<path:path>')
@cross_origin()
def send_static(path):
    """ Function to send static files from the static folder.
    """
    return send_from_directory('static', path)


@app.route('/interact/user/<user>', methods=['GET'])
@cross_origin()
def get_user(user):
    """ Function to check for existing user or create a new user folder.
    """
    if os.path.isdir(f'{app.config[INTERACT_HOME_KEY]}/projects/{user}'):
        projects = sorted(os.listdir(f'projects/{user}'))
        projects = [project for project in projects if not project.endswith('.zip')]
    else:
        os.mkdir(f'projects/{user}')
        projects = []
    response =  jsonify(message=projects)
    return response


@app.route('/interact-page/<page>', methods=['GET'])
@cross_origin()
def fetch_page_no_arguments(page):
    """ Endpoint serving the spi_serverive Home Page.
    """
    additional_args = format_header_and_footer(app.config[SERVER_ADDRESS_KEY])
    try:
        if page == 'view-queue':
            create_queue_fig(
                app.config[INTERACT_HOME_KEY],
                f'{app.config[INTERACT_HOME_KEY]}/locks/'
            )
            additional_args['queue_svg'] = safe_fetch(
                f'{app.config[INTERACT_HOME_KEY]}/locks/queue.svg'
            )
        return render_template(
            f'{page}.html',
            server_address=app.config[SERVER_ADDRESS_KEY],
            mode=app.config[MODE_KEY],
            **additional_args
        )
    except TemplateNotFound:
        return render_template(
            '404.html',
            **additional_args,
        ), 404


@app.route('/interact-page/<page>/<user>/<project>', methods=['GET'])
@cross_origin()
def fetch_page(page, user, project):
    """ Endpoint serving the spi_serverive Home Page.
    """
    header_and_footer = format_header_and_footer(app.config[SERVER_ADDRESS_KEY])
    project_home = f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'
    if not os.path.exists(f'{project_home}/core_metadata.yml'):
        try:
            return render_template(
                f'{page}.html',
                server_address=app.config[SERVER_ADDRESS_KEY],
                mode=app.config[MODE_KEY],
                user=user,
                project=project,
                **header_and_footer,
            )
        except TemplateNotFound:
            return render_template('404.html', **header_and_footer), 404

    try:
        core_config = read_meta(project_home, 'core')
        variant = core_config['variant']
    except:
        time.sleep(3)
        core_config = read_meta(project_home, 'core')
        variant = core_config['variant']

    if page == 'proteome':
        page += f'-{variant}'
    try:
        if page == 'parameters':
            html_table = generate_raw_file_table(user, project, app, variant)
            return render_template(
                f'{page}.html',
                server_address=app.config[SERVER_ADDRESS_KEY],
                mode=app.config[MODE_KEY],
                user=user,
                project=project,
                variant=variant,
                html_table=html_table,
                **header_and_footer,
            )

        return render_template(
            f'{page}.html',
            server_address=app.config[SERVER_ADDRESS_KEY],
            mode=app.config[MODE_KEY],
            user=user,
            project=project,
            variant=variant,
            **header_and_footer,
        )
    except TemplateNotFound:
        return render_template('404.html', **header_and_footer), 404


@app.route('/interact/project/<user>/<project>', methods=['GET'])
@cross_origin()
def create_project(user, project):
    """ Function to create a new user project.
    """
    if not os.path.isdir(f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'):
        os.mkdir(f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}')
    return jsonify(message='Ok')


@app.route('/interact/upload/<user>/<project>/<file_type>', methods=['POST'])
@cross_origin()
def upload_file(user, project, file_type):
    """ Function to create a new user project.
    """
    home_key= app.config[INTERACT_HOME_KEY]
    upload_home = f'{home_key}/projects/{user}/{project}/{file_type}'

    if not os.path.isdir(f'{home_key}/projects/{user}/{project}/{file_type}'):
        os.mkdir(f'projects/{user}/{project}/{file_type}')

    uploaded_files = request.files.getlist("files")

    if file_type == 'proteome':
        uploaded_files[0].save(
            f'{upload_home}/proteome_{uploaded_files[0].filename}'
        )
    elif file_type == 'proteome-select':
        uploaded_files[0].save(
            f'{upload_home}/host_{uploaded_files[0].filename}'
        )
        uploaded_files[1].save(
            f'{upload_home}/pathogen_{uploaded_files[1].filename}'
        )
        with open(
            f'{upload_home}/proteome_combined.fasta', mode='w', encoding='UTF-8',
        ) as total_file:
            for prot_file in (
                f'{upload_home}/host_{uploaded_files[0].filename}',
                f'{upload_home}/pathogen_{uploaded_files[1].filename}',
            ):
                with open(
                    prot_file, mode='r', encoding='UTF-8'
                ) as source_file:
                    for line in source_file:
                        total_file.write(line)
    elif file_type == 'search':
        # For search if there are multiple msms.txt, etc, we avoid overwriting
        for idx, u_file in enumerate(uploaded_files):
            renamed_file = f'{str(idx+1)}_' + u_file.filename.replace(" ", "_")
            u_file.save(
                f'{upload_home}/{renamed_file}'
            )
    else:
        # For raw files we preserve name
        for u_file in uploaded_files:
            u_file.save(
                f'{upload_home}/{u_file.filename}'
            )

    return jsonify(message='Ok')

@app.route('/interact/metadata', methods=['POST'])
@cross_origin()
def upload_metadata():
    """ Function for checking which files match a particular file pattern provided.
    """
    config_data = request.json
    user = config_data.pop('user')
    project = config_data.pop('project')
    metadata_type = config_data.pop('metadata_type')
    with open(
        f'projects/{user}/{project}/{metadata_type}_metadata.yml',
        'w',
        encoding='UTF-8'
    ) as yaml_out:
        yaml.dump(config_data, yaml_out)
    return jsonify(message='Ok')


@app.route('/interact/metadata/<user>/<project>/<metadata_type>', methods=['GET'])
@cross_origin()
def get_metadata(user, project, metadata_type):
    """ Function for checking which files match a particular file pattern provided.
    """
    project_home = f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'
    meta_dict = read_meta(project_home, metadata_type)
    return jsonify(message=meta_dict)


@app.route('/interact/checkPattern/<file_type>', methods=['POST'])
@cross_origin()
def check_file_pattern(file_type):
    """ Function for checking which files match a particular file pattern provided.
    """
    config_data = request.json
    user = config_data['user']
    project = config_data['project']
    project_home = f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'

    if not os.path.exists(f'{project_home}/{file_type}'):
        os.mkdir(f'{project_home}/{file_type}')
        return jsonify(message=[])

    all_files = os.listdir(f'{project_home}/{file_type}')

    return jsonify(message=all_files)


@app.route('/interact/clearPattern/<file_type>', methods=['POST'])
@cross_origin()
def clear_file_pattern(file_type):
    """ Function for checking which files match a particular file pattern provided.
    """
    config_data = request.json
    user = config_data['user']
    project = config_data['project']
    shutil.rmtree(
        f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}/{file_type}'
    )
    os.mkdir(
        f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}/{file_type}'
    )
    return jsonify(message=os.listdir(f'projects/{user}/{project}/{file_type}'))


@app.route('/interact', methods=['GET'])
@cross_origin()
def interact_landing_page():
    """ Endpoint serving the starting page.
    """
    header_and_footer = format_header_and_footer(app.config[SERVER_ADDRESS_KEY])
    usernames = sorted(os.listdir(f'{app.config[INTERACT_HOME_KEY]}/projects'))
    option_list = [
        f'<option value="{username}">{username}</option>' for username in usernames
    ]
    options_html = ' '.join(option_list)
    return render_template(
        'index.html',
        server_address=app.config[SERVER_ADDRESS_KEY],
        user_list=options_html,
        mode=app.config[MODE_KEY],
        **header_and_footer,
    )


@app.route('/interact/delete', methods=['POST'])
@cross_origin()
def delete_project():
    """ Function allow deletion of all project data.
    """
    config_dict = request.json
    user = config_dict['user']
    project = config_dict['project']
    home_key= app.config[INTERACT_HOME_KEY]
    project_home = f'{home_key}/projects/{user}/{project}'
    if os.path.exists(project_home):
        shutil.rmtree(project_home)
    return jsonify(
        message=(
            'Project deleted.'
        )
    )


@app.route('/interact/download/<user>/<project>', methods=['GET'])
@cross_origin()
def download_project(user, project):
    """ Function allow download of all project data.
    """
    home_key= app.config[INTERACT_HOME_KEY]
    project_home = f'{home_key}/projects/{user}/{project}'
    if os.path.exists(f'{project_home}/config.yml'):
        shutil.copyfile(f'{project_home}/config.yml', f'{project_home}/inspireOutput/config.yml')
    if not os.path.exists(f'{project_home}/inspireOutput'):
        os.mkdir((f'{project_home}/inspireOutput'))
    shutil.make_archive(f'{project_home}/inspireOutput', 'zip', f'{project_home}/inspireOutput')
    return send_file(f'{project_home}/inspireOutput.zip', mimetype='zip', as_attachment=True)


@app.route('/interact/cancel', methods=['POST'])
@cross_origin()
def cancel_job():
    """ Function to cancel an inSPIRE execution.
    """
    config_dict = request.json

    user = config_dict.get('user')
    project = config_dict.get('project')
    job_id = config_dict.get('jobID')
    if job_id is not None:
        job_id = int(job_id)

    cancel_message = cancel_job_helper(app.config[INTERACT_HOME_KEY], user, project, job_id)

    return jsonify(message=cancel_message)


@app.route('/interact/inspire', methods=['POST'])
@cross_origin()
def run_inspire_core():
    """ Run the inSPIRE core pipeline.
    """
    config_dict = request.json
    user = config_dict['user']
    project = config_dict['project']

    project_home = f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'
    response = jsonify(
        message=f'http://{app.config[SERVER_ADDRESS_KEY]}:5000/interact/{user}/{project}/inspire'
    )

    try:
        # If task is already running or queued, do nothing:
        if check_pids(project_home, 'inspire') == 'waiting':
            return response

        execute_inspire(app.config, project_home, config_dict)

    except Exception as err:
        return jsonify(message=f'inSPIRE-Interact failed with error code: {err}')

    return response

@app.route('/interact/<user>/<project>/<workflow>', methods=['GET'])
@cross_origin()
def check_results(user, project, workflow):
    """ Check results.
    """
    project_home = f'{app.config[INTERACT_HOME_KEY]}/projects/{user}/{project}'
    server_address = app.config[SERVER_ADDRESS_KEY]
    header_and_footer = format_header_and_footer(app.config[SERVER_ADDRESS_KEY])

    if not os.path.exists(project_home):
        return render_template('404.html', **header_and_footer), 404

    time.sleep(0.5)
    status = check_pids(project_home, workflow)

    # Task incomplete - either running or queueing (or total failure)
    if status == 'waiting':
        queue_df, task_id = fetch_queue_and_task(project_home, app.config[INTERACT_HOME_KEY])

        if not queue_df.shape[0]:
            time.sleep(2)
            queue_df, task_id = fetch_queue_and_task(
                project_home, app.config[INTERACT_HOME_KEY],
            )

        if not queue_df.shape[0]:
            return deal_with_failure(
                project_home,
                server_address,
                user,
                project,
                workflow,
                header_and_footer
            )

        if int(queue_df['taskID'].iloc[0]) == task_id:
            return deal_with_waiting(
                project_home,
                server_address,
                user,
                project,
                header_and_footer,
            )

        return deal_with_queue(
            app.config[INTERACT_HOME_KEY],
            project_home,
            server_address,
            header_and_footer,
        )

    # Task complete : either in success or failure.
    with open(f'projects/{user}/{project}/core_metadata.yml', 'r', encoding='UTF-8') as stream:
        core_config = yaml.safe_load(stream)
        variant = core_config['variant']
    if variant == 'pathogen':
        inspire_select_visible = 'visible'
        key_file = 'epitope/potentialEpitopeCandidates.xlsx'
    else:
        inspire_select_visible = 'hidden'
        key_file = 'inspire-report.html'

    if os.path.exists(f'{project_home}/inspireOutput/{key_file}'):
        return deal_with_success(
            project_home,
            server_address,
            user,
            project,
            workflow,
            inspire_select_visible,
            header_and_footer,
        )
    return deal_with_failure(
        project_home,
        server_address,
        user,
        project,
        workflow,
        header_and_footer,
    )


@app.route('/interact/get_results/<user>/<project>/<workflow>', methods=['GET'])
@cross_origin()
def get_results_file(user, project, workflow):
    """ Function to return the results files from any interact workflow.
    """
    home_key= app.config[INTERACT_HOME_KEY]
    server_address = app.config[SERVER_ADDRESS_KEY]
    project_home = f'{home_key}/projects/{user}/{project}'

    # The quantification results require a zip archive
    if workflow in ('quantification', 'inspirePathogen'):
        if os.path.exists(f'{project_home}/{ZIP_PATHS[workflow]}.zip'):
            os.remove(f'{project_home}/{ZIP_PATHS[workflow]}.zip')

        shutil.make_archive(
            f'{project_home}/{ZIP_PATHS[workflow]}',
            'zip',
            f'{project_home}/{ZIP_PATHS[workflow]}',
        )

    # The following results require the relevant results file to be sent:
    if workflow in (
        'epitopePlots',
        'psms',
        'peptides',
        'inspireLog',
        'inspirePathogen',
        'quantification'
    ):
        if os.path.exists(f'{project_home}/{KEY_FILES[workflow]}'):
            kwargs = {}
            if KEY_FILES[workflow].endswith('.zip'):
                kwargs['mimetype'] = 'zip'
                kwargs['as_attachment'] = True
            return send_file(f'{project_home}/{KEY_FILES[workflow]}', **kwargs)

    # The following send html reports
    if workflow in ('epitopeReport', 'performance', 'quantReport'):
        if (contents := safe_fetch(f'{project_home}/{KEY_FILES[workflow]}')):
            if workflow == 'epitopeReport':
                contents = contents.replace(
                    f'{project_home}/inspireOutput/epitope/spectralPlots.pdf',
                    f'http://{server_address}:5000/interact/get_results/{user}/{project}/epitopePlots',
                )

            return contents

    return Response()

def get_arguments():
    """ Function to collect command line arguments.

    Returns
    -------
    args : argparse.Namespace
        The parsed command line arguments.
    """
    parser = ArgumentParser(description='inSPIRE-Interactive Helper.')

    parser.add_argument(
        '--config_file',
        required=True,
        help='All configurations.',
    )
    parser.add_argument(
        '--mode',
        default='server',
        required=False,
    )

    return parser.parse_args()


def main():
    """ Main function to run inSPIRE-interactive.
    """
    args = get_arguments()

    with open(args.config_file, 'r', encoding='UTF-8') as stream:
        config_dict = yaml.safe_load(stream)
    for config_key in config_dict:
        if config_key not in ALL_CONFIG_KEYS:
            raise ValueError(f'Unrecognised key {config_key} found in config file.')

    app.config[INTERACT_HOME_KEY] = os.getcwd()
    app.config[INTERACT_HOME_KEY] = app.config[INTERACT_HOME_KEY].replace(
        '\\', '/'
    )
    if not os.path.exists('projects'):
        os.mkdir('projects')
    if not os.path.exists('locks'):
        os.mkdir('locks')
    if not os.path.exists(QUEUE_PATH.format(
        home_key=app.config[INTERACT_HOME_KEY])
    ):
        empty_queue_df = pd.DataFrame({
            'user': [],
            'project': [],
            'taskID': [],
            'status': [],
        })
        empty_queue_df.to_csv(QUEUE_PATH.format(
            home_key=app.config[INTERACT_HOME_KEY]), index=False
        )

    if args.mode == 'local':
        host_name = socket.gethostname()
        app.config[SERVER_ADDRESS_KEY] = host_name
    elif args.mode == 'server':
        host_name = socket.gethostname()
        app.config[SERVER_ADDRESS_KEY] = socket.gethostbyname(host_name + ".local")
    else:
        raise ValueError(f'Unknown mode {args.mode} requested')

    app.config[MHCPAN_KEY] = config_dict.get(MHCPAN_KEY)
    app.config[FRAGGER_PATH_KEY] = config_dict.get(FRAGGER_PATH_KEY)
    app.config[FRAGGER_MEMORY_KEY] = config_dict.get(FRAGGER_MEMORY_KEY)
    app.config[CPUS_KEY] = config_dict.get(CPUS_KEY, 1)
    app.config[SKYLINE_RUNNER_KEY] = config_dict.get(SKYLINE_RUNNER_KEY)
    app.config[RESCORE_COMMAND_KEY] = config_dict.get(RESCORE_COMMAND_KEY)
    if app.config[SKYLINE_RUNNER_KEY] is not None:
         app.config[SKYLINE_RUNNER_KEY] = app.config[SKYLINE_RUNNER_KEY].replace(
            '\\', '/'
        )

    app.config[MODE_KEY] = args.mode

    app.run(host='0.0.0.0', debug = False)

    print('Quitting, first clearing queue.')
    clear_queue(app.config[INTERACT_HOME_KEY])


if __name__ == '__main__':
    main()



""" Functions to deal with returning results.
"""
import os
import time

from flask import render_template
import pandas as pd
import plotly.graph_objects as go

from inspire_interact.constants import QUEUE_PATH
from inspire_interact.utils import safe_job_id_fetch

EPITOPE_CANDIDATE_ROUTE = 'inspireOutput/epitope/potentialEpitopeCandidates.csv'
NS_PERC_PSMS_PATH = 'inspireOutput/non_spectral.percolator.psms.txt'
NS_PERC_SEP_PSMS_PATH = 'inspireOutput/non_spectral.percolatorSeparate.psms.txt'
PERC_PSMS_PATH = 'inspireOutput/final.percolator.psms.txt'
PERC_SEP_PSMS_PATH = 'inspireOutput/final.percolatorSeparate.psms.txt'
QUANT_FILE_PATH = 'inspireOutput/quant/quantified_per_file.csv'



def fetch_queue_and_task(project_home, home_key):
    """ Function to fetch interact queue and the job ID of an
        inSPIRE execution.
    """
    job_id = safe_job_id_fetch(project_home)

    if not job_id:
        time.sleep(3)
        job_id = safe_job_id_fetch(project_home)
    queue_df = pd.read_csv(
        QUEUE_PATH.format(home_key=home_key)
    )
    if not queue_df.shape[0]:
        time.sleep(2)
        queue_df = pd.read_csv(
            QUEUE_PATH.format(home_key=home_key)
        )
    return queue_df, job_id


def create_queue_fig(interact_home, project_home):
    """ Function to create an svg plot of the inSPIRE-interactive queue.
    """
    queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
    queue_length = queue_df.shape[0]
    task_colors = []
    for idx in range(4):
        task_colors.append([])
        for row_idx in range(queue_length):
            if row_idx > 0:
                task_colors[idx].append('#AFEEEE')
            else:
                task_colors[idx].append('#FFE4B5')

    fig = go.Figure(
        data=[
            go.Table(
                header={
                    'height': 25,
                    'values': [
                        'User',
                        'Project',
                        'Job ID',
                        'Task Status',
                    ],
                    'line_color': 'black',
                    'align': 'left',
                },
                cells={
                    'height': 20,
                    'values': [
                        queue_df.user,
                        queue_df.project,
                        queue_df.taskID,
                        queue_df.status,
                    ],
                    'fill_color': task_colors,
                    'line_color': 'black',
                    'align': 'left',
                },
                columnwidth = [200, 200, 100, 300],
            )
        ]
    )
    total_height = (queue_df.shape[0]*20) + 25 + 4
    fig.update_layout(
        width=800,
        height=total_height+20,
        margin={'r':30, 'l':30, 't':10, 'b':10}
    )

    fig.write_image(f'{project_home}/queue.svg', engine='kaleido')

def create_status_fig(project_home):
    """ Function to create an svg plot of the status of all tasks within
        an inSPIRE job.
    """
    if not os.path.exists(f'{project_home}/taskStatus.csv'):
        return
    task_df = pd.read_csv(f'{project_home}/taskStatus.csv')
    task_colors = []
    for idx in range(3):
        task_colors.append([])
        for task_status in task_df['status'].tolist():
            if task_status == 'Queued':
                task_colors[idx].append('#AFEEEE')
            if task_status == 'Completed':
                task_colors[idx].append('darkseagreen')
            if task_status == 'Failed':
                task_colors[idx].append('lightcoral')
            if task_status == 'Running':
                task_colors[idx].append('#FFE4B5')
            if task_status == 'Skipped':
                task_colors[idx].append('#FFE4B5')
            if task_status == 'Job Cancelled':
                task_colors[idx].append('#dec8d1')

    fig = go.Figure(
        data=[
            go.Table(
                header={
                    'height': 25,
                    'values': [
                        'Task Index',
                        'Task',
                        'Task Status',
                    ],
                    'line_color': 'black',
                    'align': 'left',
                },
                cells={
                    'height': 20,
                    'values': [
                        task_df.taskIndex,
                        task_df.taskName,
                        task_df.status,
                    ],
                    'fill_color': task_colors,
                    'line_color': 'black',
                    'align': 'left',
                },
                columnwidth = [120,200,120],
            )
        ]
    )
    total_height = (task_df.shape[0]*20) + 25 + 4
    padding = (300 - total_height)/2
    fig.update_layout(
        width=500,
        height=300,
        margin={'r':30, 'l':30, 't':padding, 'b':padding}
    )

    fig_html = fig.to_html()
    fig.write_image(
        f'{project_home}/progress.svg', engine='kaleido'
    )
    with open(
        f'{project_home}/progress.html',
        mode='w',
        encoding='UTF-8',
    ) as html_out:
        html_out.write(fig_html)

def safe_fetch(file_path):
    """ Function to check if a file_path exists and return the contents
        if so.
    """
    if os.path.exists(file_path):
        with open(file_path, 'r', encoding='UTF-8') as file_contents:
            return file_contents.read()
    return ''

def deal_with_queue(interact_home, project_home, server_address, header_and_footer):
    """ Function to provide information if the inSPIRE execution is still queued.
    """
    create_queue_fig(interact_home, project_home)
    queue_svg = safe_fetch(f'{project_home}/queue.svg')
    return render_template(
        'queued.html',
        server_address=server_address,
        queue_svg=queue_svg,
        **header_and_footer
    )

def deal_with_failure(project_home, server_address, user, project, workflow, header_and_footer):
    """ Function to provide information if the inSPIRE execution has failed.
    """
    create_status_fig(project_home)
    progress_html = safe_fetch(f'{project_home}/progress.html')

    return render_template(
        'failed.html',
        server_address=server_address,
        user=user,
        project=project,
        workflow=workflow,
        progress_html=progress_html,
        **header_and_footer,
    )

def deal_with_waiting(project_home, server_address, user, project, header_and_footer):
    """ Function to return waiting screen if data is still processing.
    """
    create_status_fig(project_home)
    progress_svg = safe_fetch(f'{project_home}/progress.svg')
    return render_template(
        'waiting.html',
        progress_html=progress_svg,
        project=project,
        server_address=server_address,
        user=user,
        **header_and_footer,
    )

def deal_with_success(
        project_home,
        server_address,
        user,
        project,
        workflow,
        inspire_select_visible,
        header_and_footer,
    ):
    """ Function to return the results screen if inSPIRE has executed successfully
    """
    # Fetch the plots to be shown on results page.
    create_status_fig(project_home)
    progress_svg = safe_fetch(f'{project_home}/progress.svg')
    psm_fdr_svg = safe_fetch(f'{project_home}/inspireOutput/img/psm_fdr_curve.svg')
    ep_bar = safe_fetch(f'{project_home}/inspireOutput/img/epitope_bar_plot.svg')
    quant_svg = safe_fetch(f'{project_home}/inspireOutput/img/peptide_volcano.svg')
    if not quant_svg:
        quant_svg = safe_fetch(f'{project_home}/inspireOutput/img/norm_correlation.svg')

    return render_template(
        'ready.html',
        server_address=server_address,
        user=user,
        project=project,
        workflow=workflow,
        inspire_select_visible=inspire_select_visible,
        psm_fdr_html=psm_fdr_svg,
        ep_bar=ep_bar,
        quant_html=quant_svg,
        progress_html=progress_svg,
        inspire_increase=get_inspire_increase(project_home, 'total'),
        pathogen_increase=get_inspire_increase(project_home, 'pathogen'),
        inspire_quantified_count=get_quant_count(project_home),
        **header_and_footer
    )


def get_inspire_increase(project_home, variant):
    """ Function to calculate the percentage increase in peptides/PSMs by inSPIRE.
    """
    if variant == 'total':
        try:
            if os.path.exists(f'{project_home}/{PERC_PSMS_PATH}'):
                inspire_df = pd.read_csv(f'{project_home}/{PERC_PSMS_PATH}', sep='\t')
            elif os.path.exists(f'{project_home}/{PERC_SEP_PSMS_PATH}'):
                inspire_df = pd.read_csv(f'{project_home}/{PERC_SEP_PSMS_PATH}', sep='\t')
            else:
                inspire_df = pd.DataFrame({'q-value': []})
        except pd.errors.EmptyDataError:
            inspire_df = pd.DataFrame({'q-value': []})
        inspire_count = len(inspire_df[inspire_df['q-value'] < 0.01])

        try:
            if os.path.exists(f'{project_home}/{NS_PERC_PSMS_PATH}'):
                ns_df = pd.read_csv(f'{project_home}/{NS_PERC_PSMS_PATH}', sep='\t')
            elif os.path.exists(f'{project_home}/{NS_PERC_SEP_PSMS_PATH}'):
                ns_df = pd.read_csv(f'{project_home}/{NS_PERC_SEP_PSMS_PATH}', sep='\t')
            else:
                ns_df = pd.DataFrame({'q-value': []})
        except pd.errors.EmptyDataError:
            ns_df = pd.DataFrame({'q-value': []})
        ns_count = len(ns_df[ns_df['q-value'] < 0.01])

        if ns_count <= 0:
            return ''
        increase = round(100*(inspire_count - ns_count)/ns_count, 2)

        return f'inSPIRE increased overall PSM yield by {increase}% at 1% FDR.'

    if not os.path.exists(f'{project_home}/{EPITOPE_CANDIDATE_ROUTE}'):
        return 'No epitope candidates found.'
    pathogen_df = pd.read_csv(f'{project_home}/{EPITOPE_CANDIDATE_ROUTE}')
    shared_count = len(pathogen_df[
        pathogen_df['foundBySearchEngine'] == 'Yes'
    ])
    inspire_count = len(pathogen_df[
        pathogen_df['foundBySearchEngine'] == 'No'
    ])
    if shared_count == 0:
        increase = '>100'
    else:
        increase = round(100*inspire_count/shared_count, 2)

    return f'inSPIRE increased pathogen peptide yield by {increase}%.'


def get_quant_count(project_home):
    """ Function to get the number of peptides quantified via Skyline.
    """
    if os.path.exists(f'{project_home}/{QUANT_FILE_PATH}'):
        quant_df = pd.read_csv(f'{project_home}/{QUANT_FILE_PATH}')
        quant_count = quant_df.shape[0]
    else:
        quant_count = 0

    return f'inSPIRE quantified {quant_count} peptides via Skyline.'



""" Utility functions for inSPIRE-interact.
"""
from copy import deepcopy
import os
import time

import pandas as pd
import psutil
import yaml

from inspire_interact.constants import (
    INTERACT_HOME_KEY,
    SERVER_ADDRESS_KEY,
    TASKS_NAMES,
    TASK_DESCRIPTIONS,
)
from inspire_interact.html_snippets import (
    INSPIRE_FOOTER,
    INSPIRE_HEADER,
)

def generate_raw_file_table(user, project, app, variant):
    """ Function to create a html table with raw file, biological sample input
        and (if required) checkbox for file is infected or not
    """
    home_key= app.config[INTERACT_HOME_KEY]
    project_home = f'{home_key}/projects/{user}/{project}'

    if variant == 'pathogen':
        html_table = '''
            <tr align="center" valign="center">
                <td><b>Raw File</b></td>
                <td><b>Biological Sample</b></td>
                <td><b>Infected Sample</b></td>
            </tr>
        '''
    else:
        html_table = '''
            <tr align="center" valign="center">
                <td><b>Raw File</b></td>
                <td><b>Biological Sample</b></td>
            </tr>
        '''

    sample_files = sorted(list({
        file_name[:-4] for file_name in os.listdir(
            f'{project_home}/ms'
        ) if (
            file_name.lower().endswith('.raw') or
            file_name.lower().endswith('.mgf')
        )
    }))

    for sample_idx, sample_name in enumerate(sample_files):
        html_table += f'''
            <tr align="center" valign="center">
            <td>{sample_name}</td>
			<td>
                <input type="text" class="sample-value" value="{sample_idx+1}"
                    onkeypress="textSubmit(event, '{app.config[SERVER_ADDRESS_KEY]}', 'noAction')"/>
            </td>
        '''
        if variant == 'pathogen':
            html_table += f'''
                <td>
					<input type="checkbox" class="infection-checkbox" style="align: center" id="{sample_name}_infected" name="{sample_name}_infected">
				</td>
            '''
        html_table += '</tr>'

    return html_table


def safe_job_id_fetch(project_home):
    """ Fetch the job ID of an inSPIRE job if it exists.
    """
    if not os.path.exists(f'{project_home}/inspire_pids.txt'):
        time.sleep(2)

    if os.path.exists(f'{project_home}/inspire_pids.txt'):
        with open(f'{project_home}/inspire_pids.txt', 'r', encoding='UTF-8') as pid_file:
            return int(pid_file.readline().strip())
    return 0

def get_pids(project_home, workflow):
    """ Function to get the pids
    """
    if not os.path.exists(f'{project_home}/{workflow}_pids.txt'):
        time.sleep(3)
        if not os.path.exists(f'{project_home}/{workflow}_pids.txt'):
            return None

    with open(f'{project_home}/{workflow}_pids.txt', 'r', encoding='UTF-8') as file:
        lines = file.readlines()
        pids = [line.rstrip() for line in lines]
    return pids


def read_meta(project_home, meta_type):
    """ Function for reading metadata from a project home.
    """
    if os.path.exists(f'{project_home}/{meta_type}_metadata.yml'):
        with open(
            f'{project_home}/{meta_type}_metadata.yml',
            'r',
            encoding='UTF-8',
        ) as stream:
            return yaml.safe_load(stream)
    return {}


def check_pids(project_home, workflow):
    """ Function to check if process IDs are still running.
    """
    pids = get_pids(project_home, workflow)
    if pids is None:
        return 'clear'
    if psutil.pid_exists(int(pids[0])):
        return 'waiting'

    return 'done'


def subset_tasks(inspire_settings):
    """ Function to subset all possible inSPIRE tasks to fetch
        the ones relevant for a given job.
    """
    tasks = deepcopy(TASKS_NAMES)
    if not inspire_settings['fragger']:
        tasks = [task for task in tasks if task != 'fragger']
    if not inspire_settings['binding']:
        tasks = [task for task in tasks if task != 'predictBinding']
    if not inspire_settings['pathogen']:
        tasks = [task for task in tasks if task != 'extractCandidates']
    if not inspire_settings['quantify']:
        tasks = [task for task in tasks if task != 'quantify']
    return tasks

def write_task_status(inspire_settings, project_home):
    """ Function to write the task status DataFrame. 
    """
    tasks = subset_tasks(inspire_settings)
    task_names = [TASK_DESCRIPTIONS[task] for task in tasks]
    task_df = pd.DataFrame({
        'taskId': tasks,
        'taskName': task_names
    })
    task_df['taskIndex'] = task_df.index + 1
    task_df['status'] = 'Queued'
    task_df.to_csv(f'{project_home}/taskStatus.csv', index=False)

def format_header_and_footer(server_address):
    """ Helper function to add the server address to the inSPIRE header and footer.
    """
    return {
        'inspire_header': INSPIRE_HEADER.format(
            server_address=server_address,
        ),
        'inspire_footer': INSPIRE_FOOTER.format(
            server_address=server_address,
        ),
    }



""" Functions for executing inSPIRE within inSPIRE-interactive.
"""
import ast
import os
import sys
import time

from inspire.config import ALL_CONFIG_KEYS
import yaml

from inspire_interact.constants import (
    CPUS_KEY,
    FRAGGER_MEMORY_KEY,
    FRAGGER_PATH_KEY,
    INTERACT_HOME_KEY,
    MHCPAN_KEY,
    RESCORE_COMMAND_KEY,
    SKYLINE_RUNNER_KEY,
)
from inspire_interact.utils import (
    check_pids, write_task_status, read_meta, subset_tasks,
)
from inspire_interact.inspire_script import INSPIRE_SCRIPT



def execute_inspire(app_config, project_home, config_dict):
    """ Function to execute inSPIRE, writes config file, a bash file with all
        required tasks, and then executes the bash file in the background.
    """
    inspire_settings = prepare_inspire(config_dict, project_home, app_config)
    write_task_status(inspire_settings, project_home)

    # In case of rerunning, we should be careful not to reuse this file.
    if os.path.exists(f'{project_home}/inspireOutput/formated_df.csv'):
        os.remove(f'{project_home}/inspireOutput/formated_df.csv')

    task_list = subset_tasks(inspire_settings)
    inspire_script = INSPIRE_SCRIPT.format(
        home_key=app_config[INTERACT_HOME_KEY],
        project_home=project_home,
        task_list=','.join(task_list)
    )
    script_path = f'{project_home}/inspire_script.py'
    with open(script_path, mode='w', encoding='UTF-8') as script_file:
        script_file.writelines(inspire_script)

    os.popen(
        f'{sys.executable} {script_path} > {project_home}/inspire_log.txt 2>&1'
    )
    for idx in range(3):
        if check_pids(project_home, 'inspire') == 'waiting':
            break
        
        time.sleep(idx+1)


def prepare_inspire(config_dict, project_home, app_config):
    """ Function to prepare the inSPIRE run.
    """
    inspire_settings = {
        'fragger': False,
        'pathogen': False,
    }

    inspire_settings['quantify'] = bool(config_dict['runQuantification'])

    output_config = {
        'experimentTitle': config_dict['project'],
        'scansFormat': 'mgf',
        'outputFolder': f'{project_home}/inspireOutput',
        'mzAccuracy': float(config_dict['mzAccuracy']),
        'ms1Accuracy': float(config_dict['ms1Accuracy']),
        'mzUnits': config_dict['mzUnits'],
        'silentExecution': True,
        'reuseInput': True,
        'fraggerPath': app_config[FRAGGER_PATH_KEY],
        'fraggerMemory': app_config[FRAGGER_MEMORY_KEY],
        'nCores': app_config[CPUS_KEY],
        'skylineRunner': app_config[SKYLINE_RUNNER_KEY],
        RESCORE_COMMAND_KEY: app_config[RESCORE_COMMAND_KEY],
        'technicalReplicates': config_dict['technicalReplicates'],
    }

    meta_dict = read_meta(project_home, 'search')
    if not meta_dict:
        raise ValueError('No Search Engine information was provided, check this screen.')

    output_config['searchEngine'] = meta_dict.get('searchEngine', 'msfragger')
    inspire_settings['fragger'] = bool(meta_dict.get('runFragger', 1))

    if 'useBindingAffinity' in config_dict:
        output_config['useBindingAffinity'] = config_dict['useBindingAffinity']
        output_config['alleles'] = [
            elem.strip() for elem in  config_dict["alleles"].split(",")
        ]
        output_config['netMHCpan'] = app_config[MHCPAN_KEY]
        inspire_settings['binding'] = True
    else:
        inspire_settings['binding'] = False

    if not inspire_settings['fragger']:
        output_config['searchResults'] = [
            f'{project_home}/search/{filename}' for filename in os.listdir(
                f'{project_home}/search/'
            )
        ]

    output_config['scansFolder'] = f'{project_home}/ms'

    if output_config['searchEngine'] in ('mascot', 'msfragger'):
        output_config['rescoreMethod'] = 'percolatorSeparate'
    else:
        output_config['rescoreMethod'] = 'percolator'

    if os.path.exists(f'{project_home}/proteome'):
        proteome_files = os.listdir(f'{project_home}/proteome')
        if len(proteome_files) > 0:
            prot_file_name = proteome_files[0]
            output_config['proteome'] = f'{project_home}/proteome/{prot_file_name}'
            output_config['inferProteins'] = True
    else:
        proteome_files = os.listdir(f'{project_home}/proteome-select')
        if len(proteome_files) < 2:
            raise ValueError('For inSPIRE-Pathogen, proteome files must be uploaded.')
        inspire_settings['pathogen'] = True
        output_config['proteome'] = f'{project_home}/proteome-select/proteome_combined.fasta'
        path_name = [
            prot_file for prot_file in proteome_files if prot_file.startswith('pathogen_')
        ][0]
        host_name = [
            prot_file for prot_file in proteome_files if prot_file.startswith('host_')
        ][0]
        output_config['pathogenProteome'] = f'{project_home}/proteome-select/{path_name}'
        output_config['hostProteome'] = f'{project_home}/proteome-select/{host_name}'
        output_config['controlFlags'] = [
            elem.strip() for elem in config_dict["controlFlags"].split(",") if elem
        ]
        output_config['inferProteins'] = True

    for config_key, config_value in config_dict['additionalConfigs'].items():
        if config_key not in ALL_CONFIG_KEYS:
            raise ValueError(f'Error {config_key} is not a valid inSPIRE config.')
        try:
            output_config[config_key] = ast.literal_eval(config_value)
        except:
            output_config[config_key] = config_value


    with open(f'{project_home}/config.yml', 'w', encoding='UTF-8') as yaml_out:
        yaml.dump(output_config, yaml_out)

    return inspire_settings



""" Constants for the inspire_interact package.
"""
INTERACT_HOME_KEY = 'interactHome'
SERVER_ADDRESS_KEY = 'serverAddress'
FRAGGER_PATH_KEY = 'fraggerPath'
FRAGGER_MEMORY_KEY = 'fraggerMemory'
CPUS_KEY = 'maxInspireCpus'
MHCPAN_KEY = 'netMHCpan'
MODE_KEY = 'mode'
SKYLINE_RUNNER_KEY = 'skylineRunner'
RESCORE_COMMAND_KEY = 'rescoreCommand'

QUEUE_PATH = '{home_key}/locks/inspireQueue.csv'

ALL_CONFIG_KEYS = [
    CPUS_KEY,
    FRAGGER_PATH_KEY,
    FRAGGER_MEMORY_KEY,
    INTERACT_HOME_KEY,
    MHCPAN_KEY,
    RESCORE_COMMAND_KEY,
    SERVER_ADDRESS_KEY,
    SKYLINE_RUNNER_KEY,
]

INTERMEDIATE_FILES = [
    # 'input_all_features.tab'
]

KEY_FILES = {
    'epitopePlots': 'inspireOutput/epitope/spectralPlots.pdf',
    'epitopeReport': 'inspireOutput/epitope/inspire-epitope-report.html',
    'psms': 'inspireOutput/finalPsmAssignments.csv',
    'peptides': 'inspireOutput/finalPeptideAssignments.csv',
    'inspireLog': 'inspire_log.txt',
    'inspirePathogen': 'inspireOutput/epitope.zip',
    'performance': 'inspireOutput/inspire-report.html',
    'quantification': 'inspireOutput/quant.zip',
    'quantReport': 'inspireOutput/quant/inspire-quant-report.html',
}

ZIP_PATHS = {
    'quantification': 'inspireOutput/quant',
    'inspirePathogen': 'inspireOutput/epitope',
}

TASKS_NAMES = [
    'convert',
    'fragger',
    'prepare',
    'predictSpectra',
    'predictBinding',
    'featureGeneration',
    'featureSelection+',
    'generateReport',
    'quantify',
    'extractCandidates',
]
TASK_DESCRIPTIONS = {
    'convert': 'Converting MS Data',
    'fragger': 'Executing MSFragger',
    'prepare': 'Preparing results',
    'predictSpectra': 'Predicting spectra',
    'predictBinding': 'Predicting binding',
    'featureGeneration': 'Generating features',
    'featureSelection+': 'Executing rescoring',
    'generateReport': 'Creating report',
    'quantify': 'Quantifying peptides',
    'extractCandidates': 'Finding pathogen peptides',
}



""" Scripts for managing inSPIRE jobs running on after another.
"""
from argparse import ArgumentParser
import os
from time import sleep

import pandas as pd

from inspire_interact.constants import QUEUE_PATH

def get_arguments():
    """ Function to collect command line arguments.

    Returns
    -------
    args : argparse.Namespace
        The parsed command line arguments.
    """
    parser = ArgumentParser(description='inSPIRE-Interactive Helper.')

    parser.add_argument(
        '--interact_home',
        required=True,
        help='All configurations.',
    )
    parser.add_argument(
        '--project_home',
        required=True,
        help='All configurations.',
    )
    parser.add_argument(
        '--queue_task',
        required=True,
    )
    parser.add_argument(
        '--inspire_task',
        required=False,
    )
    parser.add_argument(
        '--inspire_status',
        required=False,
    )

    return parser.parse_args()


def add_to_queue(project_home, interact_home):
    """ Function to add an inSPIRE job to the inSPIRE-Interactive queue.
    """
    with open(f'{project_home}/inspire_pids.txt', 'r', encoding='UTF-8') as pid_file:
        task_id = int(pid_file.readline().strip())
    if os.path.exists(QUEUE_PATH.format(home_key=interact_home)):
        queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
    else:
        queue_df = pd.DataFrame({
            'user': [],
            'project': [],
            'taskID': [],
            'status': [],
        })

    user = project_home.split('/')[-2]
    project = project_home.split('/')[-1]
    append_df = pd.DataFrame({
        'user': [user],
        'project': [project],
        'taskID': [task_id],
        'status': 'waiting',
    })
    queue_df = pd.concat([queue_df, append_df])
    queue_df.to_csv(QUEUE_PATH.format(home_key=interact_home), index=False)

def remove_from_queue(interact_home, job_id):
    """ Function to remove an inSPIRE job from the inSPIRE interactive queue.
    """
    queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
    if queue_df[queue_df['taskID'] == job_id].shape[0]:
        drop_index = queue_df[queue_df['taskID'] == job_id].index[0]
        queue_df = queue_df.drop(drop_index, axis=0)
        queue_df.to_csv(QUEUE_PATH.format(home_key=interact_home), index=False)

def update_status(project_home, interact_home, inspire_task, inspire_status):
    """ Function to update the status of a task in the taskStatus file.
    """
    task_df = pd.read_csv(f'{project_home}/taskStatus.csv')
    queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
    if inspire_task == 'start':
        task_df['status'].iloc[0] = 'Running'
        queue_df.iloc[
            0, queue_df.columns.get_loc('status')
        ] = task_df.iloc[
            0, task_df.columns.get_loc('taskName')
        ]
    else:
        index = task_df.index[task_df['taskId'] == inspire_task].tolist()[0]
        if inspire_status == '0':
            task_df['status'].iloc[index] = 'Completed'
            if index + 1 < len(task_df):
                task_df['status'].iloc[index + 1] = 'Running'
                queue_df.iloc[0, queue_df.columns.get_loc('status')] = task_df.iloc[
                    index + 1, task_df.columns.get_loc('taskName')
                ]
        else:
            task_df['status'].iloc[index] = 'Failed'
            for following_idx in range(index+1, len(task_df)):
                task_df['status'].iloc[following_idx] = 'Skipped'

    task_df.to_csv(f'{project_home}/taskStatus.csv', index=False)
    queue_df.to_csv(QUEUE_PATH.format(home_key=interact_home), index=False)


def check_queue(project_home, interact_home):
    """ Function to check if the job is first in the queue.
    """
    with open(f'{project_home}/inspire_pids.txt', 'r', encoding='UTF-8') as pid_file:
        task_id = int(pid_file.readline().strip())

    while True:
        queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
        if int(queue_df['taskID'].iloc[0]) == task_id:
            break
        sleep(60)



""" Script for running inSPIRE as a string to be formatted.
"""

INSPIRE_SCRIPT = """
import os
import sys
import traceback

from inspire.run import run_inspire

from inspire_interact.queue_manager import (
    add_to_queue,
    check_queue,
    remove_from_queue,
    update_status,
)

def execute_taks(home_key, project_home, task_list, proc_id):
    ''' Function which runs in the background, running inSPIRE tasks and
    interacting with inSPIRE-interactive queue.
    '''
    task_list = task_list.split(',')
    config_file = '{project_home}/config.yml'
    add_to_queue(project_home, home_key)
    check_queue(project_home, home_key)
    update_status(
        project_home,
        home_key,
        'start',
        '0',
    )

    for task in task_list:
        try:
            run_inspire(task, config_file)
            status = '0'
        except Exception as e:
            print('inSPIRE failed on task' + task  + 'with Exception:' + str(e))
            print(traceback.format_exc())
            status = '1'

        update_status(
            project_home,
            home_key,
            task,
            status,
        )
        if status != '0':
            if task not in ('predictBinding', 'quantify', 'generateReport'):
                remove_from_queue(home_key, proc_id)
                sys.exit(0)
    remove_from_queue(home_key, proc_id)

if __name__ == '__main__':
    proc_id = os.getpid()
    with open(f'{project_home}/inspire_pids.txt', 'w', encoding='UTF-8') as pid_file:
        pid_file.writelines(str(proc_id)+'\\n')
    execute_taks('{home_key}', '{project_home}', '{task_list}', proc_id)
    sys.exit(0)

"""



from setuptools import setup, find_packages

setup(
    name='inspire-interact',
    version='1.0rc2',
    description='Interactive GUI for inSPIRE Platform.',
    author='John Cormican, Sahil Khan, Juliane Liepe, Manuel S. Pereira',
    author_email='juliane.liepe@mpinat.mpg.de',
    include_package_data=True,
    long_description=open('README.md', mode='r', encoding='UTF-8').read(),
    long_description_content_type='text/markdown',
    py_modules=[
        'inspire_interact',
    ],
    entry_points={
        'console_scripts': [
            'inspire-interact=inspire_interact.api:main',
        ]
    },
    packages=find_packages(),
    python_requires='>=3.11',
    install_requires=[
        'inspirems==2.0rc3',
        'blinker==1.6.2',
        'click==8.1.3',
        'flask==2.3.2',
        'flask_cors==3.0.10',
        'itsdangerous==2.1.2',
        'psutil==5.9.6',
        'Werkzeug==3.0.1',
    ],
    project_urls={
        'Homepage': 'https://github.com/QuantSysBio/inSPIRE',
        'Tracker': 'https://github.com/QuantSysBio/inSPIRE/issues',
    },
    zip_safe=False,
)



""" HTML Snippets to be injected throughout
"""


INSPIRE_HEADER = """
    <dev class  = "header">
        <dev class = "header-left">
            
            <img src="http://{server_address}:5000/static/inSPIRE-logo-nobg.png"
            class = "inSPIRE-logo-nobg" 
            width = "180px"
            onclick="goHome('{server_address}')"/>
        </dev>
        
        <!-- Header Center-->
        <dev class = "home">
            <a href=http://{server_address}:5000/interact-page/home> <p class = "home-text"> Home </p>
            </a>
        </dev>

        <dev class = "about-us">
            <a href=http://{server_address}:5000/interact-page/about> <p class = "about-us-text"> About </p>
            </a>
        </dev>

        <dev class = "contact">
            <a href=http://{server_address}:5000/interact-page/contact> <p class = "contact-text"> Contact </p>
            </a>
        </dev>

        <dev class = "FAQ">
            <a href=http://{server_address}:5000/interact-page/faq> <p class = "FAQ-text"> FAQ </p>
            </a>
        </dev>

        <dev class = "view-queue">
            <a href=http://{server_address}:5000/interact-page/view-queue> <p class = "view-queue-text">Job Queue</p>
            </a>
        </dev>

        <dev class = "header-right">

            <button type="button" class = "button-git"  
                onclick="location.href='https://github.com/QuantSysBio/inSPIRE';">
                <p>&nbsp;&nbsp;GIT HUB</p>
            </button>
        

       
            <button type="button" class = "button"
                onclick="location.href='http://{server_address}:5000/interact';">
                <p>&nbsp;&nbsp;&nbsp;&nbsp;GET STARTED</p>
            </button>
        </dev>

    </dev>
"""

INSPIRE_FOOTER = """
    <div class = "bottom">
        <div class = "footer">
            <p>
                <b>
                    Disclaimer: inSPIRE and inSPIRE-interactive are available
                    under an open source license. Check licenses if installing
                    additional supported tools.<br>
                    If you use this tool, please don't forget 
                    to cite related <a href = "http://{server_address}:5000/interact-page/references">articles</a>.
                </b>
            </p>
        </div>
    </div>
"""



""" Scripts for cleaning up after cancelations/failures.
"""
import os
import signal

import pandas as pd

from inspire_interact.constants import QUEUE_PATH
from inspire_interact.queue_manager import remove_from_queue
from inspire_interact.utils import get_pids


def get_user_and_project(home_key, job_id):
    """ Function to get the user and project name from a job ID.
    """
    queue_df = pd.read_csv(
        QUEUE_PATH.format(home_key=home_key)
    )
    queue_df = queue_df[
        queue_df['taskID'] == job_id
    ]
    if len(queue_df):
        return queue_df['user'].iloc[0], queue_df['project'].iloc[0]
    return None, None

def clear_queue(interact_home):
    """ Function to cancel all running jobs and clear the queue.
    """
    if os.path.exists(QUEUE_PATH.format(home_key=interact_home)):
        queue_df = pd.read_csv(QUEUE_PATH.format(home_key=interact_home))
        for _, df_row in queue_df.iterrows():
            cancel_job_helper(interact_home, df_row['user'], df_row['project'], df_row['taskID'])


def cancel_job_helper(home_key, user, project, job_id):
    """ Function to cancel a job and delete it from the queue.
    """
    if user is None:
        user, project = get_user_and_project(home_key, job_id)
    project_home = f'{home_key}/projects/{user}/{project}'

    if job_id is None:
        pids = get_pids(project_home, 'inspire')
    else:
        pids = [int(job_id)]

    if pids is None:
        return 'No task was running. Please refresh the page.'

    remove_from_queue(home_key, int(pids[0]))

    task_killed = False
    for pid in pids:
        try:
            os.kill(int(pid), signal.SIGTERM)
            task_killed = True
        except OSError:
            continue

    if not task_killed:
        return 'No task was running. Please refresh the page.'

    task_df = pd.read_csv(f'{project_home}/taskStatus.csv')
    task_df['status'] = 'Job Cancelled'
    task_df.to_csv(f'{project_home}/taskStatus.csv', index=False)

    return 'Task cancelled. Please refresh the page.'
